package hamiltonianGraph;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Stack;

// vertexy a hrany "na pozadí" - prohledávací algorimus
public class Controller {

    Generator generator;
    ArrayList<Vertex> vertexList;
    ArrayList<Edge> lineList;

    int vertexCount = 6;       //výchozí počet vertxů
    boolean exist = true;      // existence ham. grafu
    boolean InStepflag = false;      //true pokud nalezena spojnice mezi body
    int[][] path;              //pole hran
    int[][] vertexDuplicateChecker;
    int visitCount = 0;         //počet navštívených uzlů
    int[] result;
    int backStepFlag = -2;
    Stack<Integer> stack = new Stack<>();
    ArrayList<Vertex> solvedVertexes = new ArrayList<>(); //list výsledných uzlů ham. grafu

    public Controller() {
        init();
    }


    public void init() {
        generator = new Generator();
        vertexList = new ArrayList<>();
        lineList = new ArrayList<>();
        stack = new Stack<>();
        visitCount = 0;
        exist = true;
        result = new int[vertexCount + 30];
    }

    // generování uzlů - automatické
    public void generate() {

        initTables();
        int x, y;
        for (int i = 0; i < vertexCount; i++) {
            x = generator.getRandomIntegerBetweenRange(0, 10);
            y = generator.getRandomIntegerBetweenRange(0, 10);

            if (vertexDuplicateChecker[x][y] == 0) {
                vertexDuplicateChecker[x][y] = 1;
                Vertex vertex = new Vertex(x, y);
                vertexList.add(vertex);
            } else {
                i--;
            }
        }
        generateLines();
    }

    // generování uzlů s vlastními uzly
    public void generateCustom(List<Node> nodes, List<Edge> edges) {

        vertexCount = nodes.size();
        initTables();

        for (int i = 0; i < nodes.size(); i++) {
            vertexList.add(new Vertex(i, i));
            nodes.get(i).setVertex(new Vertex(i, i));
        }
        customLines(nodes, edges);
    }

    // generování hran s vlastními hranami
    private void customLines(List<Node> nodes, List<Edge> edges) {

        int m = 0;
        int n = 0;
        Node n1, n2;


        for (Edge edge : edges) {
            n1 = edge.getN1();
            n2 = edge.getN2();

            for (int j = 0; j < nodes.size(); j++) {
                if (nodes.get(j) == n1) {
                    m = j;
                } else if (nodes.get(j) == n2) {
                    n = j;
                }
            }
            path[m][n] = 1;
            path[n][m] = 1;
        }
    }

    public void initTables() {
        path = new int[vertexCount][vertexCount];
        for (int[] row : path) {
            Arrays.fill(row, 0);
        }

        //velikost podle generatoru, nyni 0 - 9
        vertexDuplicateChecker = new int[30][30];
        for (int[] row : vertexDuplicateChecker) {
            Arrays.fill(row, 0);
        }
    }


    // generování hran - automatické
    private void generateLines() {
        //úplný graf - max hran
        int max = (vertexCount * (vertexCount - 1)) / 2;

        int mode;
        if (vertexCount < 4) {
            mode = generator.getRandomIntegerBetweenRange(1, 14);
        } else if (vertexCount < 6) {
            mode = generator.getRandomIntegerBetweenRange(1, 11);
        } else if (vertexCount < 7) {
            mode = generator.getRandomIntegerBetweenRange(1, 9);
        } else {
            mode = generator.getRandomIntegerBetweenRange(1, 8);
        }

        //počet hran poodle vybraneho modu
        int numberOfLines;
        if (mode < 3) {
            numberOfLines = generator.getRandomIntegerBetweenRange(0, max / 2);

        } else if (mode > 3 && mode < 6) {
            numberOfLines = generator.getRandomIntegerBetweenRange(max / 2, max * 2 / 3);

        } else {
            numberOfLines = generator.getRandomIntegerBetweenRange(max * 2 / 3, max);
        }

        for (
                int i = 0;
                i < numberOfLines; i++) {
            int m = generator.getRandomIntegerBetweenRange(0, vertexCount - 1);
            int n = generator.getRandomIntegerBetweenRange(0, vertexCount - 1);

            if ((m != n) && (path[m][n] != 1) && (path[n][m] != 1)) {

                path[m][n] = 1;
                path[n][m] = 1; // vytvori symetrickou tabulku hran
            } else {
                i--;
            }
        }
    }

    // prohledávání grafu - řídící funkce
    public boolean search() {

        int i = 0;
        int j = 0;
        vertexList.get(0).setVisited(true);
        solvedVertexes.add(new Vertex(vertexList.get(0).getX(), vertexList.get(0).getY()));
            rowInspect(i, j);

            if (exist) {
                //graf existuje
                solvedVertexes.add(new Vertex(vertexList.get(0).getX(), vertexList.get(0).getY()));
                return true;
            } else {
                //graf neexistuje
                return false;
            }
    }

    // prohledávání grafu - rekurzivní funkce
    private void rowInspect(int i, int j) {
        InStepflag = false;

        //zkouška jestli je kam pokračovat
        while (j < vertexCount) {
            if ((path[i][j] == 1) && (!vertexList.get(j).isVisited())) {
                InStepflag = true;
                break;
            }
            // ošetření, aby se "neodešlo" z tabulky hran
            if (j != vertexCount - 1) {
                j++;
            } else {
                break;
            }
        }

        //přenastavení hodnot, při návratu
        if (backStepFlag != -2) {
            vertexList.get(backStepFlag).setVisited(false);
            backStepFlag = -2;
            visitCount--;
        }

        // je možné zanoření
        if (InStepflag) {
            stack.push(i);
            stack.push(j);
            vertexList.get(j).setVisited(true);

            solvedVertexes.add(new Vertex(vertexList.get(j).getX(), vertexList.get(j).getY()));
            visitCount++;
            rowInspect(j, 0);

            // kontrola existence poslední hrany vedoucí do výchozího uzlu
        } else if ((visitCount + 1 == vertexCount) && (path[i][0] == 1)) {

        } else {
            // návrat
            if (!stack.isEmpty()) {
                backStepFlag = i;

                j = stack.pop();
                i = stack.pop();

                if (solvedVertexes.size() != 0) {
                    solvedVertexes.remove(solvedVertexes.size() - 1);
                } rowInspect(i, j);

            } else {
                //ham. graf neexistuje
                exist = false;
            }
        }
    }

    public void setVertexCount(int vertexCount) {
        this.vertexCount = vertexCount;
    }
}
