package hamiltonianGraph;

import java.awt.*;
import java.util.List;

//grafická reprezentace uzlu
public class Node {

    private Point p;
    private int r;
    private Color color;
    private boolean selected = false;
    private Rectangle rectangle = new Rectangle();
    private Vertex vertex;

    public Node(Point p, int r, Color color, Vertex vertex) {
        this.p = p;
        this.r = r;
        this.color = color;
        this.vertex = vertex;
        setBoundary(rectangle);
    }

    public Node(Point p, int r, Color color) {
        this.p = p;
        this.r = r;
        this.color = color;
        setBoundary(rectangle);
    }

    public void setColor(Color color) {
        this.color = color;
    }

    public Vertex getVertex() {
        return vertex;
    }

    public void setVertex(Vertex vertex) {
        this.vertex = vertex;
    }

    // výpočet hranice výběrového obdélníku
    private void setBoundary(Rectangle b) {
        b.setBounds(p.x - r, p.y - r, 2 * r, 2 * r);
    }

    public void draw(Graphics g) {
        g.setColor(this.color);
        g.fillOval(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
        if (selected) {
            g.setColor(Color.darkGray);
            g.drawRect(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
        }
    }
    //vrátí označené uzly
    public static void getSelected(List<Node> list, List<Node> selected) {
        selected.clear();
        for (Node n : list) {
            if (n.isSelected()) {
                selected.add(n);
            }
        }
    }

    public static void selectNone(List<Node> list) {
        for (Node n : list) {
            n.setSelected(false);
        }
    }

    public static boolean selectOne(List<Node> list, Point p) {
        for (Node n : list) {
            if (n.contains(p)) {
                if (!n.isSelected()) {
                    Node.selectNone(list);
                    n.setSelected(true);
                }
                return true;
            }
        }
        return false;
    }

    // výběr uzlu z označovacího obdélníku
    public static void selectRectangle(List<Node> list, Rectangle r) {
        for (Node n : list) {
            n.setSelected(r.contains(n.p));
        }
    }

    //změna pozice
    public static void updatePosition(List<Node> list, Point d) {
        for (Node n : list) {
            if (n.isSelected()) {
                n.p.x += d.x;
                n.p.y += d.y;
                n.setBoundary(n.rectangle);
            }
        }
    }

    //změna poloměru uzlu
    public static void updateRadius(List<Node> list, int r) {
        for (Node n : list) {
            if (n.isSelected()) {
                n.r = r;
                n.setBoundary(n.rectangle);
            }
        }
    }

    public Point getLocation() {
        return p;
    }

    public boolean contains(Point p) {
        return rectangle.contains(p);
    }

    public boolean isSelected() {
        return selected;
    }

    public void setSelected(boolean selected) {
        this.selected = selected;
    }
}